DECLARE @PhysicalHost sysname,
	@DBName sysname,
	@BackupPath nvarchar(100),
	@MaxID int,
	@CurrID int,
	@RestoreSQL nvarchar(max),
	@RestoreFiles nvarchar(max),
	@MaxFile int,
	@CurrFile int,
	@BFName nvarchar(260),
	@BFNumber int,
	@ErrNumber int,
	@ErrSeverity int,
	@ErrState int,
	@ErrProcedure sysname,
	@ErrLine int,
	@ErrMsg nvarchar(2048);
Declare @DBs Table (DBID int identity(1, 1) not null primary key,
			DBName sysname,
			BackupPath nvarchar(100));
Declare @BackupFiles Table (BFID int identity(1, 1) not null primary key,
			BFName nvarchar(260),
			BFNumber int);

-- Get the name of the instance where the backups are run
Select @PhysicalHost = data_source
From sys.servers
Where name = N'LITESPEEDLSSOURCE';

-- Get the location of each backup for the last backup created
With BUD
As (Select RowID = ROW_NUMBER() Over (Partition By BS.database_name Order By BS.backup_finish_date Desc),
		BS.database_name As DatabaseName,
		PhysDevice = Reverse(Right(Reverse(BMF.physical_device_name), Len(BMF.physical_device_name) - CharIndex('\', Reverse(BMF.physical_device_name)) + 1))
	From LITESPEEDLSSOURCE.msdb.dbo.backupset As BS
	Inner Join LITESPEEDLSSOURCE.msdb.dbo.backupmediafamily As BMF On BMF.media_set_id = BS.media_set_id)
Insert Into @DBs (DBName, BackupPath)
Select DatabaseName,
	PhysDevice
From BUD
Where RowID = 1;

-- Set the start and stop points for the database
Select @MaxID = Max(DBID),
	@CurrID = 1
From @DBs;

-- Loop through each database
While @CurrID <= @MaxID
  Begin
	-- Get important values for current database
	Select @DBName = DBName,
		@BackupPath = BackupPath
	From @DBs
	Where DBID = @CurrID;

	-- Only seeding databases that don't already exist
	If Not Exists (Select * From sys.databases
					Where name = @DBName)
	  Begin
		Delete From @BackupFiles;

		-- Get list of files to restore for database
		Insert Into @BackupFiles(BFName, BFNumber)
		EXEC master.dbo.xp_restore_automated @database = @DBName,
			@backuppath = @BackupPath,
			@backupextension = N'bak',
			@checksubfolders = 1,
			@sourceserver = @PhysicalHost,
			@sourcedatabase = @DBName,
			@backuptype = N'tlog',
			@DontUseCopyOnly = 0,
			@affinity = 0,
			@logging = 0,
			@withreplace = 0,
			@dryrun = 1;

		Set @RestoreFiles = Null;

		-- Create the restore database command (accounts for multiple backup files)
		Select @RestoreFiles = IsNull(@RestoreFiles + char(10), '') + N'@filename = ''' + BFName + N''','
		From @BackupFiles
		Where BFName Like N'%full%.bak';

		Set @RestoreSQL = N'EXEC master.dbo.xp_restore_database @database = N''' + @DBName + ''',
					' + @RestoreFiles + N'
					@affinity = 0,
					@logging = 0,
					@with = N''STATS = 10'',
					@with = N''NORECOVERY'';';

		-- Restore the full backup
		Exec sys.sp_executesql @RestoreSQL;

		-- Loop through remaining files
		Select @MaxFile = Max(BFID),
			@CurrFile = Min(BFID)
		From @BackupFiles
		Where BFName Not Like N'%full%.bak';

		While @CurrFile <= @MaxFile
		  Begin
			Select @BFName = BFName,
				@BFNumber = BFNumber
			From @BackupFiles
			Where BFID = @CurrFile;

			-- Restore each log file
			Begin Try
				EXEC master.dbo.xp_restore_log @database = @DBName,
					@filename = @BFName,
					@filenumber = @BFNumber,
					@affinity = 0,
					@logging = 0,
					@with = N'STATS = 10',
					@with = N'NORECOVERY';
			End Try
			Begin Catch
				Set @ErrNumber = ERROR_NUMBER();
				Set @ErrSeverity = ERROR_SEVERITY();
				Set @ErrState = ERROR_STATE();
				Set @ErrProcedure = ERROR_PROCEDURE();
				Set @ErrLine = ERROR_LINE();
				Set @ErrMsg = ERROR_MESSAGE();
				RaisError(@ErrMsg, 1, 1);
			End Catch

			-- Next loop
			Set @CurrFile = @CurrFile + 1;
		  End
	  End
	-- Next loop
	Set @CurrID = @CurrID + 1;
  End
